@@ -8,6 +8,7 @@ import time
 import weakref
 from collections import deque
 from enum import Enum
+from functools import cached_property
 from pathlib import Path, PurePath
 from typing import (
     Any,
@@ -25,6 +26,7 @@ from typing import (
     Sequence,
     Set,
     Tuple,
+    TypedDict,
     Union,
     cast,
 )
@@ -33,7 +35,6 @@ from robot.api.parsing import get_model
 from robot.errors import VariableError
 from robot.output import LOGGER
 from robot.running import EXECUTION_CONTEXTS, Keyword, TestCase, TestSuite
-from robot.utils import NormalizedDict
 from robot.variables import evaluate_expression
 
 from robotcode.core.event import event
@@ -68,6 +69,7 @@ from .dap_types import (
     Variable,
     VariablePresentationHint,
 )
+from .id_manager import IdManager
 
 if get_robot_version() >= (7, 0):
     from robot.running import UserKeyword as UserKeywordHandler
@@ -96,6 +98,19 @@ class Undefined:
 UNDEFINED = Undefined()
 
 
+class DebugRepr(reprlib.Repr):
+    def __init__(self) -> None:
+        super().__init__()
+        self.maxtuple = 50
+        self.maxlist = 50
+        self.maxarray = 50
+        self.maxdict = 500
+        self.maxset = 50
+        self.maxfrozenset = 50
+        self.maxdeque = 50
+        self.maxstring = 500
+
+
 class EvaluateResult(NamedTuple):
     result: str
     type: Optional[str] = None
@@ -147,8 +162,15 @@ class StackTraceResult(NamedTuple):
 
 
 class InvalidThreadIdError(Exception):
-    def __init__(self, thread_id: Any) -> None:
-        super().__init__(f"Invalid thread id {thread_id}")
+    def __init__(self, current_thread_id: Any, expected_thread_id: Any = None) -> None:
+        super().__init__(
+            f"Invalid thread id {current_thread_id}"
+            + (f", expected {expected_thread_id}" if expected_thread_id is not None else "")
+        )
+
+
+class MarkerObject:
+    pass
 
 
 class StackFrameEntry:
@@ -181,10 +203,10 @@ class StackFrameEntry:
         self.libname = libname
         self.kwname = kwname
         self.longname = longname
-        self._suite_marker = object()
-        self._test_marker = object()
-        self._local_marker = object()
-        self._global_marker = object()
+        self._suite_marker = MarkerObject()
+        self._test_marker = MarkerObject()
+        self._local_marker = MarkerObject()
+        self._global_marker = MarkerObject()
         self.stack_frames: Deque[StackFrameEntry] = deque()
 
     def __repr__(self) -> str:
@@ -195,21 +217,27 @@ class StackFrameEntry:
             return self.stack_frames[0]
         return self
 
-    @property
+    _id_manager = IdManager()
+
+    @cached_property
     def id(self) -> int:
-        return id(self)
+        return self._id_manager.get_id(self)
 
+    @cached_property
     def test_id(self) -> int:
-        return id(self._test_marker)
+        return self._id_manager.get_id(self._test_marker)
 
+    @cached_property
     def suite_id(self) -> int:
-        return id(self._suite_marker)
+        return self._id_manager.get_id(self._suite_marker)
 
+    @cached_property
     def local_id(self) -> int:
-        return id(self._local_marker)
+        return self._id_manager.get_id(self._local_marker)
 
+    @cached_property
     def global_id(self) -> int:
-        return id(self._global_marker)
+        return self._id_manager.get_id(self._global_marker)
 
 
 class HitCountEntry(NamedTuple):
@@ -256,6 +284,15 @@ else:
             self.steps.pop()
 
 
+breakpoint_id_manager = IdManager()
+
+
+class ExceptionInformation(TypedDict):
+    text: Optional[str]
+    description: str
+    status: str
+
+
 class Debugger:
     __instance: ClassVar[Optional["Debugger"]] = None
     __lock: ClassVar = threading.RLock()
@@ -329,6 +366,7 @@ class Debugger:
         self._evaluate_cache: List[Any] = []
         self._variables_cache: Dict[int, Any] = {}
         self._variables_object_cache: List[Any] = []
+        self._current_exception: Optional[ExceptionInformation] = None
 
     @property
     def state(self) -> State:
@@ -394,12 +432,12 @@ class Debugger:
         with self.condition:
             self.state = State.Stopped
 
-            if self.main_thread is not None and self.main_thread.ident:
+            if self.main_thread_is_alive:
                 self.send_event(
                     self,
                     ContinuedEvent(
                         body=ContinuedEventBody(
-                            thread_id=self.main_thread.ident,
+                            thread_id=self.main_thread_id,
                             all_threads_continued=True,
                         )
                     ),
@@ -407,21 +445,24 @@ class Debugger:
 
             self.condition.notify_all()
 
+    def check_thread_id(self, thread_id: int) -> None:
+        if not self.main_thread_is_alive and thread_id != self.main_thread_id:
+            raise InvalidThreadIdError(thread_id, self.main_thread_id)
+
     def continue_all(self) -> None:
-        if self.main_thread is not None and self.main_thread.ident is not None:
-            self.continue_thread(self.main_thread.ident)
+        if self.main_thread_is_alive:
+            self.continue_thread(self.main_thread_id)
 
     def continue_thread(self, thread_id: int) -> None:
-        if self.main_thread is None or thread_id != self.main_thread.ident:
-            raise InvalidThreadIdError(thread_id)
+        self.check_thread_id(thread_id)
 
         with self.condition:
             self.requested_state = RequestedState.Running
             self.condition.notify_all()
 
     def pause_thread(self, thread_id: int) -> None:
-        if self.main_thread is None or thread_id != self.main_thread.ident:
-            raise InvalidThreadIdError(thread_id)
+        if thread_id != 0:
+            self.check_thread_id(thread_id)
 
         with self.condition:
             self.requested_state = RequestedState.Pause
@@ -429,8 +470,7 @@ class Debugger:
             self.condition.notify_all()
 
     def next(self, thread_id: int, granularity: Optional[SteppingGranularity] = None) -> None:
-        if self.main_thread is None or thread_id != self.main_thread.ident:
-            raise InvalidThreadIdError(thread_id)
+        self.check_thread_id(thread_id)
 
         with self.condition:
             if self.full_stack_frames and self.full_stack_frames[0].type in [
@@ -464,8 +504,7 @@ class Debugger:
         target_id: Optional[int] = None,
         granularity: Optional[SteppingGranularity] = None,
     ) -> None:
-        if self.main_thread is None or thread_id != self.main_thread.ident:
-            raise InvalidThreadIdError(thread_id)
+        self.check_thread_id(thread_id)
 
         with self.condition:
             self.requested_state = RequestedState.StepIn
@@ -473,8 +512,7 @@ class Debugger:
             self.condition.notify_all()
 
     def step_out(self, thread_id: int, granularity: Optional[SteppingGranularity] = None) -> None:
-        if self.main_thread is None or thread_id != self.main_thread.ident:
-            raise InvalidThreadIdError(thread_id)
+        self.check_thread_id(thread_id)
 
         with self.condition:
             self.requested_state = RequestedState.StepOut
@@ -509,6 +547,7 @@ class Debugger:
         lines: Optional[List[int]] = None,
         source_modified: Optional[bool] = None,
     ) -> List[Breakpoint]:
+
         if self.is_windows_path(source.path or ""):
             path: pathlib.PurePath = pathlib.PureWindowsPath(source.path or "")
         else:
@@ -523,7 +562,7 @@ class Debugger:
             )
             return [
                 Breakpoint(
-                    id=id(v),
+                    id=breakpoint_id_manager.get_id(v),
                     source=Source(path=str(path)),
                     verified=True,
                     line=v.line,
@@ -536,6 +575,9 @@ class Debugger:
         return []
 
     def process_start_state(self, source: str, line_no: int, type: str, status: str) -> None:
+        if self.state == State.CallKeyword:
+            return
+
         if self.state == State.Stopped:
             return
 
@@ -547,8 +589,9 @@ class Debugger:
                 self,
                 StoppedEvent(
                     body=StoppedEventBody(
+                        description="Paused",
                         reason=StoppedReason.PAUSE,
-                        thread_id=threading.current_thread().ident,
+                        thread_id=self.main_thread_id,
                     )
                 ),
             )
@@ -562,8 +605,9 @@ class Debugger:
                     self,
                     StoppedEvent(
                         body=StoppedEventBody(
+                            description="Next step",
                             reason=StoppedReason.STEP,
-                            thread_id=threading.current_thread().ident,
+                            thread_id=self.main_thread_id,
                         )
                     ),
                 )
@@ -576,8 +620,9 @@ class Debugger:
                 self,
                 StoppedEvent(
                     body=StoppedEventBody(
+                        description="Step in",
                         reason=StoppedReason.STEP,
-                        thread_id=threading.current_thread().ident,
+                        thread_id=self.main_thread_id,
                     )
                 ),
             )
@@ -590,8 +635,9 @@ class Debugger:
                 self,
                 StoppedEvent(
                     body=StoppedEventBody(
+                        description="Step out",
                         reason=StoppedReason.STEP,
-                        thread_id=threading.current_thread().ident,
+                        thread_id=self.main_thread_id,
                     )
                 ),
             )
@@ -601,6 +647,7 @@ class Debugger:
             if source_path in self.breakpoints:
                 breakpoints = [v for v in self.breakpoints[source_path].breakpoints if v.line == line_no]
                 if len(breakpoints) > 0:
+
                     for point in breakpoints:
                         if point.condition is not None:
                             hit = False
@@ -661,9 +708,10 @@ class Debugger:
                             self,
                             StoppedEvent(
                                 body=StoppedEventBody(
+                                    description="Breakpoint hit",
                                     reason=StoppedReason.BREAKPOINT,
-                                    thread_id=threading.current_thread().ident,
-                                    hit_breakpoint_ids=[id(v) for v in breakpoints],
+                                    thread_id=self.main_thread_id,
+                                    hit_breakpoint_ids=[breakpoint_id_manager.get_id(v) for v in breakpoints],
                                 )
                             ),
                         )
@@ -675,6 +723,8 @@ class Debugger:
         description: str,
         text: Optional[str],
     ) -> None:
+        if self.state == State.CallKeyword:
+            return
         if self.state == State.Stopped:
             return
 
@@ -692,12 +742,14 @@ class Debugger:
             self.requested_state = RequestedState.Nothing
             self.state = State.Paused
 
+            self._current_exception = {"text": text, "description": description, "status": status}
+
             self.send_event(
                 self,
                 StoppedEvent(
                     body=StoppedEventBody(
                         reason=reason,
-                        thread_id=threading.current_thread().ident,
+                        thread_id=self.main_thread_id,
                         description=description,
                         text=text,
                     )
@@ -732,12 +784,12 @@ class Debugger:
                 if self.requested_state == RequestedState.Running:
                     self.requested_state = RequestedState.Nothing
                     self.state = State.Running
-                    if self.main_thread is not None and self.main_thread.ident is not None:
+                    if self.main_thread_is_alive:
                         self.send_event(
                             self,
                             ContinuedEvent(
                                 body=ContinuedEventBody(
-                                    thread_id=self.main_thread.ident,
+                                    thread_id=self.main_thread_id,
                                     all_threads_continued=True,
                                 )
                             ),
@@ -745,6 +797,7 @@ class Debugger:
                     continue
 
                 break
+            self._current_exception = None
 
     def start_output_group(self, name: str, attributes: Dict[str, Any], type: Optional[str] = None) -> None:
         if self.group_output:
@@ -756,7 +809,7 @@ class Debugger:
                 OutputEvent(
                     body=OutputEventBody(
                         output=f"\u001b[38;5;14m{(type + ' ') if type else ''}\u001b[0m{name}\n",
-                        category=OutputCategory.CONSOLE,
+                        category="OutputCategory.CONSOLE",
                         group=OutputGroup.START,
                         source=Source(path=str(self.map_path_to_client(source))) if source else None,
                         line=line_no if source is not None else None,
@@ -778,7 +831,8 @@ class Debugger:
                         category=OutputCategory.CONSOLE,
                         group=OutputGroup.END,
                         source=Source(path=str(self.map_path_to_client(source))) if source else None,
-                        line=line_no,
+                        line=line_no if source is not None else None,
+                        column=0 if source is not None else None,
                     )
                 ),
             )
@@ -864,6 +918,9 @@ class Debugger:
                 self.stack_frames[0].stack_frames.popleft()
 
     def start_suite(self, name: str, attributes: Dict[str, Any]) -> None:
+        if self.state == State.CallKeyword:
+            return
+
         if not self.run_started:
             self.run_started = True
             self.debug_logger = DebugLogger()
@@ -892,7 +949,7 @@ class Debugger:
                     StoppedEvent(
                         body=StoppedEventBody(
                             reason=StoppedReason.ENTRY,
-                            thread_id=threading.current_thread().ident,
+                            thread_id=self.main_thread_id,
                         )
                     ),
                 )
@@ -909,6 +966,9 @@ class Debugger:
                 self.wait_for_running()
 
     def end_suite(self, name: str, attributes: Dict[str, Any]) -> None:
+        if self.state == State.CallKeyword:
+            return
+
         if self.debug:
             status = attributes.get("status", "")
 
@@ -928,6 +988,9 @@ class Debugger:
         self.remove_stackframe_entry(name, type, source, line_no)
 
     def start_test(self, name: str, attributes: Dict[str, Any]) -> None:
+        if self.state == State.CallKeyword:
+            return
+
         source = attributes.get("source")
         line_no_dummy = attributes.get("lineno", 1)
         if isinstance(line_no_dummy, str):
@@ -952,13 +1015,16 @@ class Debugger:
             self.wait_for_running()
 
     def end_test(self, name: str, attributes: Dict[str, Any]) -> None:
+        if self.state == State.CallKeyword:
+            return
+
         if self.debug:
             status = attributes.get("status", "")
 
             if status == "FAIL":
                 self.process_end_state(
                     status,
-                    {"failed_test"},
+                    {""},
                     "Test failed.",
                     f"Test failed{f': {v}' if (v := attributes.get('message')) else ''}",
                 )
@@ -983,6 +1049,9 @@ class Debugger:
             return EXECUTION_CONTEXTS.current.namespace.get_runner(name)._handler
 
     def start_keyword(self, name: str, attributes: Dict[str, Any]) -> None:
+        if self.state == State.CallKeyword:
+            return
+
         status = attributes.get("status", "")
         source = attributes.get("source")
         line_no_dummy = attributes.get("lineno", 1)
@@ -1110,6 +1179,9 @@ class Debugger:
         return True
 
     def end_keyword(self, name: str, attributes: Dict[str, Any]) -> None:
+        if self.state == State.CallKeyword:
+            return
+
         type = attributes.get("type")
         if self.debug:
             status = attributes.get("status", "")
@@ -1123,7 +1195,7 @@ class Debugger:
                             {"uncaught_failed_keyword"}
                             if self.is_not_caughted_by_keyword()
                             and self.is_not_caugthed_by_except(self.last_fail_message)
-                            else {}
+                            else []
                         ),
                     },
                     "Keyword failed.",
@@ -1151,20 +1223,30 @@ class Debugger:
     def set_main_thread(self, thread: threading.Thread) -> None:
         self.main_thread = thread
 
-    def get_threads(self) -> List[Thread]:
-        main_thread = self.main_thread or threading.main_thread()
+    @property
+    def main_thread_id(self) -> int:
+        return 1 if self.main_thread_is_alive else 0
 
-        return [
-            Thread(
-                id=main_thread.ident if main_thread.ident else 0,
-                name=main_thread.name or "",
-            )
-        ]
+    @property
+    def main_thread_is_alive(self) -> bool:
+        return self.main_thread is not None and self.main_thread.is_alive()
+
+    def get_threads(self) -> List[Thread]:
+        return (
+            [
+                Thread(
+                    id=self.main_thread_id,
+                    name="RobotMain",
+                )
+            ]
+            if self.main_thread_is_alive
+            else []
+        )
 
     WINDOW_PATH_REGEX: ClassVar = re.compile(r"^(([a-z]:[\\/])|(\\\\)).*$", re.RegexFlag.IGNORECASE)
 
     @classmethod
-    def is_windows_path(cls, path: Union[os.PathLike[str], str]) -> bool:
+    def is_windows_path(cls, path: Union["os.PathLike[str]", str]) -> bool:
         return bool(cls.WINDOW_PATH_REGEX.fullmatch(str(path)))
 
     @staticmethod
@@ -1174,7 +1256,7 @@ class Debugger:
         except ValueError:
             return None
 
-    def map_path_to_client(self, path: Union[os.PathLike[str], str]) -> pathlib.PurePath:
+    def map_path_to_client(self, path: Union["os.PathLike[str]", str]) -> pathlib.PurePath:
         if not isinstance(path, PurePath):
             path = PurePath(path)
 
@@ -1211,8 +1293,7 @@ class Debugger:
         levels: Optional[int] = None,
         format: Optional[StackFrameFormat] = None,
     ) -> StackTraceResult:
-        if self.main_thread is None or thread_id != self.main_thread.ident:
-            raise InvalidThreadIdError(thread_id)
+        self.check_thread_id(thread_id)
 
         start_frame = start_frame or 0
         levels = start_frame + (levels or len(self.stack_frames))
@@ -1317,11 +1398,8 @@ class Debugger:
         level = message["level"]
         current_frame = self.full_stack_frames[0] if self.full_stack_frames else None
 
-        if (
-            self.output_messages
-            or current_frame is not None
-            and current_frame.type != "KEYWORD"
-            and level in ["FAIL", "ERROR", "WARN"]
+        if self.output_messages or (
+            current_frame is not None and current_frame.type != "KEYWORD" and level in ["FAIL", "ERROR", "WARN"]
         ):
             self._send_log_event(message["timestamp"], level, message["message"], "messages")
 
@@ -1336,7 +1414,7 @@ class Debugger:
                         name="Local",
                         expensive=False,
                         presentation_hint="local",
-                        variables_reference=entry.local_id(),
+                        variables_reference=entry.local_id,
                     )
                 )
                 if context.variables._test is not None and entry.type == "KEYWORD":
@@ -1345,7 +1423,7 @@ class Debugger:
                             name="Test",
                             expensive=False,
                             presentation_hint="test",
-                            variables_reference=entry.test_id(),
+                            variables_reference=entry.test_id,
                         )
                     )
                 if context.variables._suite is not None and entry.type in [
@@ -1357,7 +1435,7 @@ class Debugger:
                             name="Suite",
                             expensive=False,
                             presentation_hint="suite",
-                            variables_reference=entry.suite_id(),
+                            variables_reference=entry.suite_id,
                         )
                     )
                 if context.variables._global is not None:
@@ -1366,28 +1444,37 @@ class Debugger:
                             name="Global",
                             expensive=False,
                             presentation_hint="global",
-                            variables_reference=entry.global_id(),
+                            variables_reference=entry.global_id,
                         )
                     )
 
         return result
 
-    def _new_cache_id(self) -> int:
-        o = object()
-        self._variables_object_cache.append(o)
-        return id(o)
+    _cache_id_manager = IdManager()
 
-    def _create_variable(self, name: str, value: Any) -> Variable:
+    def _new_cache_id(self) -> int:
+        o = MarkerObject()
+        self._variables_object_cache.append(o)
+        return StackFrameEntry._id_manager.get_id(o)
+
+    debug_repr = DebugRepr()
+
+    def _create_variable(
+        self, name: str, value: Any, presentation_hint: Optional[VariablePresentationHint] = None
+    ) -> Variable:
         if isinstance(value, Mapping):
             v_id = self._new_cache_id()
             self._variables_cache[v_id] = value
             return Variable(
                 name=name,
-                value=reprlib.repr(value),
+                value=self.debug_repr.repr(value),
                 type=repr(type(value)),
                 variables_reference=v_id,
                 named_variables=len(value) + 1,
                 indexed_variables=0,
+                presentation_hint=(
+                    presentation_hint if presentation_hint is not None else VariablePresentationHint(kind="data")
+                ),
             )
 
         if isinstance(value, Sequence) and not isinstance(value, str):
@@ -1395,14 +1482,15 @@ class Debugger:
             self._variables_cache[v_id] = value
             return Variable(
                 name=name,
-                value=reprlib.repr(value),
+                value=self.debug_repr.repr(value),
                 type=repr(type(value)),
                 variables_reference=v_id,
                 named_variables=1,
                 indexed_variables=len(value),
+                presentation_hint=VariablePresentationHint(kind="data"),
             )
 
-        return Variable(name=name, value=repr(value), type=repr(type(value)))
+        return Variable(name=name, value=self.debug_repr.repr(value), type=repr(type(value)))
 
     if get_robot_version() >= (7, 0):
 
@@ -1422,44 +1510,56 @@ class Debugger:
         count: Optional[int] = None,
         format: Optional[ValueFormat] = None,
     ) -> List[Variable]:
-        result: MutableMapping[str, Any] = NormalizedDict(ignore="_")
+        result: MutableMapping[str, Any] = {}
 
         if filter is None:
             entry = next(
                 (
                     v
                     for v in self.stack_frames
-                    if variables_reference in [v.global_id(), v.suite_id(), v.test_id(), v.local_id()]
+                    if variables_reference in [v.global_id, v.suite_id, v.test_id, v.local_id]
                 ),
                 None,
             )
             if entry is not None:
                 context = entry.context()
                 if context is not None:
-                    if entry.global_id() == variables_reference:
+                    if entry.global_id == variables_reference:
                         result.update(
                             {k: self._create_variable(k, v) for k, v in context.variables._global.as_dict().items()}
                         )
-                    elif entry.suite_id() == variables_reference:
+                    elif entry.suite_id == variables_reference:
                         globals = context.variables._global.as_dict()
+                        vars = entry.get_first_or_self().variables()
+                        vars_dict = vars.as_dict() if vars is not None else {}
                         result.update(
                             {
                                 k: self._create_variable(k, v)
                                 for k, v in context.variables._suite.as_dict().items()
-                                if k not in globals or globals[k] != v
+                                if (k not in globals or globals[k] != v) and (k in vars_dict)
                             }
                         )
-                    elif entry.test_id() == variables_reference:
+                    elif entry.test_id == variables_reference:
                         globals = context.variables._suite.as_dict()
+                        vars = entry.get_first_or_self().variables()
+                        vars_dict = vars.as_dict() if vars is not None else {}
                         result.update(
                             {
                                 k: self._create_variable(k, v)
                                 for k, v in context.variables._test.as_dict().items()
-                                if k not in globals or globals[k] != v
+                                if (k not in globals or globals[k] != v) and (k in vars_dict)
                             }
                         )
-                    elif entry.local_id() == variables_reference:
+                    elif entry.local_id == variables_reference:
                         vars = entry.get_first_or_self().variables()
+
+                        if self._current_exception is not None:
+                            result["${EXCEPTION}"] = self._create_variable(
+                                "${EXCEPTION}",
+                                self._current_exception,
+                                VariablePresentationHint(kind="virtual"),
+                            )
+
                         if vars is not None:
                             p = entry.parent() if entry.parent else None
 
@@ -1518,7 +1618,7 @@ class Debugger:
 
                 padding = len(str(len(value)))
 
-                for i, v in enumerate(value, start or 0):
+                for i, v in enumerate(value[start:], start or 0):
                     result[str(i)] = self._create_variable(str(i).zfill(padding), v)
                     c += 1
                     if count is not None and c >= count:
@@ -1537,8 +1637,8 @@ class Debugger:
 
         return list(result.values())
 
-    IS_VARIABLE_RE: ClassVar = re.compile(r"^[$@&%]\{.*\}(\[[^\]]*\])?$")
-    IS_VARIABLE_ASSIGNMENT_RE: ClassVar = re.compile(r"^[$@&%]\{.*\}=?$")
+    IS_VARIABLE_RE: ClassVar = re.compile(r"^[$@&]\{.*\}(\[[^\]]*\])?$")
+    IS_VARIABLE_ASSIGNMENT_RE: ClassVar = re.compile(r"^[$@&]\{.*\}=?$")
     SPLIT_LINE: ClassVar = re.compile(r"(?= {2,}| ?\t)\s*")
     CURRDIR: ClassVar = re.compile(r"(?i)\$\{CURDIR\}")
 
@@ -1552,6 +1652,18 @@ class Debugger:
         def _run_keyword(self, kw: Keyword, context: Any) -> Any:
             return kw.run(context)
 
+    if get_robot_version() >= (7, 2):
+
+        @staticmethod
+        def check_message_is_logged(listener: Any, msg: Any) -> bool:
+            return cast(bool, listener._is_logged(msg))
+
+    else:
+
+        @staticmethod
+        def check_message_is_logged(listener: Any, msg: Any) -> bool:
+            return cast(bool, listener._is_logged(msg.level))
+
     def evaluate(
         self,
         expression: str,
@@ -1594,10 +1706,8 @@ class Debugger:
                 else evaluate_context.variables._global
             )
             if (
-                isinstance(context, EvaluateArgumentContext)
-                and context != EvaluateArgumentContext.REPL
-                or self.expression_mode
-            ):
+                isinstance(context, EvaluateArgumentContext) and context != EvaluateArgumentContext.REPL
+            ) or self.expression_mode:
                 if expression.startswith("! "):
                     splitted = self.SPLIT_LINE.split(expression[2:].strip())
 
@@ -1630,13 +1740,15 @@ class Debugger:
                         result = vars.replace_scalar(expression)
                     except VariableError:
                         if context is not None and (
-                            isinstance(context, EvaluateArgumentContext)
-                            and (
-                                context
-                                in [
-                                    EvaluateArgumentContext.HOVER,
-                                    EvaluateArgumentContext.WATCH,
-                                ]
+                            (
+                                isinstance(context, EvaluateArgumentContext)
+                                and (
+                                    context
+                                    in [
+                                        EvaluateArgumentContext.HOVER,
+                                        EvaluateArgumentContext.WATCH,
+                                    ]
+                                )
                             )
                             or context
                             in [
@@ -1650,8 +1762,10 @@ class Debugger:
                 else:
                     result = internal_evaluate_expression(vars.replace_string(expression), vars)
             else:
-                if self.IS_VARIABLE_RE.match(expression.strip()):
-                    result = vars[expression.strip()]
+                parts = self.SPLIT_LINE.split(expression.strip())
+                if parts and len(parts) == 1 and self.IS_VARIABLE_RE.match(parts[0].strip()):
+                    # result = vars[parts[0].strip()]
+                    result = vars.replace_scalar(parts[0].strip())
                 else:
 
                     def get_test_body_from_string(command: str) -> TestCase:
@@ -1679,18 +1793,19 @@ class Debugger:
                                         result = e
                                         break
                                     finally:
-                                        messages = LOGGER._log_message_cache or []
-                                        for msg in messages or ():
-                                            # hack to get and evaluate log level
-                                            listener: Any = next(iter(LOGGER), None)
-                                            if listener is None or listener._is_logged(msg.level):
-                                                self.log_message(
-                                                    {
-                                                        "level": msg.level,
-                                                        "message": msg.message,
-                                                        "timestamp": msg.timestamp,
-                                                    }
-                                                )
+                                        if get_robot_version() <= (7, 2):
+                                            messages = LOGGER._log_message_cache or []
+                                            for msg in messages or ():
+                                                # hack to get and evaluate log level
+                                                listener: Any = next(iter(LOGGER), None)
+                                                if listener is None or self.check_message_is_logged(listener, msg):
+                                                    self.log_message(
+                                                        {
+                                                            "level": msg.level,
+                                                            "message": msg.message,
+                                                            "timestamp": msg.timestamp,
+                                                        }
+                                                    )
                         return result
 
                     result = self.run_in_robot_thread(run_kw)
@@ -1803,7 +1918,7 @@ class Debugger:
             (
                 v
                 for v in self.full_stack_frames
-                if variables_reference in [v.global_id(), v.local_id(), v.suite_id(), v.test_id()]
+                if variables_reference in [v.global_id, v.local_id, v.suite_id, v.test_id]
             ),
             None,
         )
@@ -1838,7 +1953,7 @@ class Debugger:
                 if option.filter_id in [
                     "failed_keyword",
                     "uncaught_failed_keyword",
-                    "failed_test",
+                    "",
                     "failed_suite",
                 ]:
                     entry = ExceptionBreakpointsEntry(
